# Ubuntu CI Engine
# Copyright 2013 Canonical Ltd.

# This program is free software: you can redistribute it and/or modify it
# under the terms of the GNU Affero General Public License version 3, as
# published by the Free Software Foundation.

# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranties of
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR
# PURPOSE.  See the GNU Affero General Public License for more details.

# You should have received a copy of the GNU Affero General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import json
import urllib2

from ci_utils.ticket_states import (
    TicketWorkflowStep,
    TicketWorkflowStepStatus,
)


class TicketApi(object):

    def __init__(self, url, ticket_id):
        self._url = url + '/api/v1'
        self._ticket_id = ticket_id

    def _patch(self, resource, data):
        req = urllib2.Request(resource)
        req.add_header('Content-Type', 'application/json')
        req.get_method = lambda: 'PATCH'
        urllib2.urlopen(req, json.dumps(data))

    def _patch_pkg_status(self, status):
        step = TicketWorkflowStep.PKG_BUILDING.value
        status = status.value
        url = self._url + '/updateticketstatus/' + self._ticket_id + '/'
        self._patch(url, {'status': status, 'current_workflow_step': step})

    def set_pkg_waiting(self):
        self._patch_pkg_status(TicketWorkflowStepStatus.PKG_BUILDING_WAITING)

    def set_pkg_inprogress(self):
        self._patch_pkg_status(
            TicketWorkflowStepStatus.PKG_BUILDING_INPROGRESS)

    def set_pkg_complete(self, success=True):
        state = TicketWorkflowStepStatus.PKG_BUILDING_COMPLETED
        if not success:
            state = TicketWorkflowStepStatus.PKG_BUILDING_FAILED
        self._patch_pkg_status(state)

    def _patch_ticket_status(self, status):
        step = TicketWorkflowStep.IMAGE_BUILDING.value
        status = status.value
        url = self._url + '/updateticketstatus/' + self._ticket_id + '/'
        self._patch(url, {'status': status, 'current_workflow_step': step})

    def set_image_waiting(self):
        self._patch_ticket_status(
            TicketWorkflowStepStatus.IMAGE_BUILDING_WAITING)

    def set_image_inprogress(self):
        self._patch_ticket_status(
            TicketWorkflowStepStatus.IMAGE_BUILDING_INPROGRESS)

    def set_image_complete(self, success=True):
        state = TicketWorkflowStepStatus.IMAGE_BUILDING_COMPLETED
        if not success:
            state = TicketWorkflowStepStatus.IMAGE_BUILDING_FAILED
        self._patch_ticket_status(state)

    def _patch_testing_status(self, status):
        step = TicketWorkflowStep.IMAGE_TESTING.value
        status = status.value
        url = self._url + '/updateticketstatus/' + self._ticket_id + '/'
        self._patch(url, {'status': status, 'current_workflow_step': step})

    def set_testing_waiting(self):
        self._patch_testing_status(
            TicketWorkflowStepStatus.IMAGE_TESTING_WAITING)

    def set_testing_inprogress(self):
        self._patch_testing_status(
            TicketWorkflowStepStatus.IMAGE_TESTING_INPROGRESS)

    def set_testing_complete(self, success=True):
        state = TicketWorkflowStepStatus.IMAGE_TESTING_PASSED
        if not success:
            state = TicketWorkflowStepStatus.IMAGE_TESTING_FAILED
        self._patch_testing_status(state)
