#!/usr/bin/env python

import json
import os
import unittest

import httplib2

from tests.deployers import DEPLOYER_DIR, DeployerTest


class TestTestRunner(DeployerTest):
    """Integration tests for test runner service run on a juju deployment"""

    deployer_cfg = os.path.join(DEPLOYER_DIR, 'test-runner.yaml')

    def post_test_image(self, test_request_id, image_id, package_list):
        url = 'http://{}:{}/api/v1/test_image'.format(
            *self.get_ip_and_port('tr-restish'))
        client = httplib2.Http()
        resp, content = client.request(
            url, 'POST',
            headers={'Content-Type': 'application/json'},
            body=json.dumps({'progress_trigger': test_request_id,
                             'image_id': image_id,
                             'package_list': package_list,
                             }),
        )
        return resp, content

    def test_test_image_succeeds(self):
        resp, content = self.post_test_image('t-rex', 'foo',
                                             ['uci-tests'])
        self.assertEqual('204', resp['status'])
        self.assertEqual('', content)


if __name__ == "__main__":
    unittest.main()
